local _, BigBrother = ...

local module = BigBrother:NewModule("Checks", "AceTimer-3.0")
if not module then return end

local L = LibStub("AceLocale-3.0"):GetLocale("BigBrother")
local SpellData = BigBrother.SpellData

local function whisper(name, text)
	if not module.db.profile.blame then return end
	ChatThrottleLib:SendChatMessage("BULK", "", ("BigBrother> %s"):format(text), "WHISPER", nil, name)
end

function BigBrother:ConsumableCheck(channel)
	local groups = self:GetCheckedGroups()
	local inRaid = IsInRaid()

	local missingFlaskList = {}
	local missingElixirList = {}
	local missingFoodList = {}

	for unit, index in self:IterateGroup() do
		local name, realm = UnitName(unit)
		if not name then break end
		local shortName = name
		if realm then name = name.."-"..realm end

		local subgroup = inRaid and select(3, GetRaidRosterInfo(index)) or 1
		local online = UnitIsConnected(unit)
		local dead = UnitIsDeadOrGhost(unit)

		if online and not dead and groups[subgroup] then

			--check flasks/elixirs
			if self.db.profile.checkFlasks then
				local numElixirs = 0
				for _, spellName in next, SpellData.elixirsBattle do
					if UnitBuff(unit, spellName) then
						numElixirs = numElixirs + 1
						break
					end
				end
				for _, spellName in next, SpellData.elixirsGuardian do
					if UnitBuff(unit, spellName) then
						numElixirs = numElixirs + 1
						break
					end
				end

				if numElixirs == 1 then
					whisper(name, L["One Elixir"])
					missingElixirList[#missingElixirList + 1] = shortName
				elseif numElixirs == 0 then
					local found = nil
					for _, spellName in next, SpellData.flasks do
						if UnitBuff(unit, spellName) then
							found = true
							break
						end
					end
					if not found then
						whisper(name, L["No Flask"])
						missingFlaskList[#missingFlaskList + 1] = shortName
					end
				end
			end

			--check food
			if self.db.profile.checkFood then
				local found = nil
				for _, spellName in next, SpellData.foods do
					local id = select(11, UnitBuff(unit, spellName))
					if id and (not self.db.profile.maxOnly or SpellData.foodsMax[id]) then
						found = true
						break
					end
				end
				if not found then
					whisper(name, L["Not Well Fed"])
					missingFoodList[#missingFoodList + 1] = shortName
				end
			end

		end --if
	end --for

	if #missingElixirList > 0 then
		self:SendChatMessageList(L["One Elixir"], missingElixirList, channel)
	end
	if #missingFlaskList > 0 then
		self:SendChatMessageList(L["No Flask"], missingFlaskList, channel)
	end
	if #missingFoodList > 0 then
		self:SendChatMessageList(L["Not Well Fed"], missingFoodList, channel)
	end
end

function module:READY_CHECK(_, sender)
	-- unchecked (0), checked (1), greyed (2)
	if not IsInGroup() or IsPartyLFG() or self.db.profile.checkReadyCheck == 0 then return end

	if self.db.profile.checkReadyCheck == 2 or (self.db.profile.checkReadyCheck == 1 and UnitIsUnit(sender, "player")) then
		self:BuffCheck()
	end
end


---------------------------------------
-- Init

function module:OnInitialize()
	self.db = BigBrother.db:RegisterNamespace("Checks", {
		profile = {
			checkFlasks = true,
			checkFood = true,
			checkBuffs = true,
			outputDefault = "raid",
			blame = false,
			checkReadyCheck = 1, --started by you
			maxOnly = false,
		}
	})
end

function module:OnEnable()
	self:RegisterEvent("READY_CHECK")
end


---------------------------------------
-- Options


function module:AddOptions(options)
	options.args.check_button = {
		name = L["Buff Check"],
		desc = L["Report who is missing consumables and/or buffs."],
		type = "execute",
		func = function() BigBrother:BuffCheck() end,
		order = 5,
		width = "full",
	}

	local db = self.db.profile
	local bb = BigBrother.db.profile
	options.args.checks = {
		name = L["Checks"],
		--desc = L["Checks"],
		type = "group",
		get = function(info) return db[info[#info]] end,
		set = function(info, value) db[info[#info]] = value end,
		order = 20,
		order = 1,
		args = {
			checks = {
				name = L["Checks"],
				desc = L["Set what is included in a buff check."],
				type = "group",
				inline = true,
				order = 10,
				args = {
					checkFlasks = {
						name = L["Flasks/Elixirs"],
						desc = L["Include flasks and elixirs in checks."],
						type = "toggle",
						order = 5,
					},
					checkFood = {
						name = L["Food Buffs"],
						desc = L["Include food buffs in checks."],
						type = "toggle",
						order = 15,
					},
					checkBuffs = {
						name = L["Raid Buffs"],
						desc = L["Include raid buffs in checks."],
						type = "toggle",
						order = 20,
					},
				},
			}, --checks
			groups = {
				name = L["Checked Raid Groups"],
				desc = L["Set which raid groups are checked for buffs."],
				type = "group",
				inline = true,
				order = 20,
				disabled = function() return bb.autogroups end,
				args = {
					autogroups = {
						name = L["Use Instance Difficulty"],
						desc = L["Use the currently selected instance difficulty to determine which groups to check."],
						type = "toggle",
						get = function() return bb.autogroups end,
						set = function(info, value) bb.autogroups = value end,
						width = "full",
						order = 1,
						disabled = false,
					},
					group1 = {
						name = L["Group 1"],
						type = "toggle",
						get = function() return bb.groups[1] end,
						set = function(info, value) bb.groups[1] = value end,
						order = 11,
					},
					group2 = {
						name = L["Group 2"],
						type = "toggle",
						get = function() return bb.groups[2] end,
						set = function(info, value) bb.groups[2] = value end,
						order = 12,
					},
					group3 = {
						name = L["Group 3"],
						type = "toggle",
						get = function() return bb.groups[3] end,
						set = function(info, value) bb.groups[3] = value end,
						order = 13,
					},
					group4 = {
						name = L["Group 4"],
						type = "toggle",
						get = function() return bb.groups[4] end,
						set = function(info, value) bb.groups[4] = value end,
						order = 14,
					},
					group5 = {
						name = L["Group 5"],
						type = "toggle",
						get = function() return bb.groups[5] end,
						set = function(info, value) bb.groups[5] = value end,
						order = 15,
					},
					group6 = {
						name = L["Group 6"],
						type = "toggle",
						get = function() return bb.groups[6] end,
						set = function(info, value) bb.groups[6] = value end,
						order = 16,
					},
					group7 = {
						name = L["Group 7"],
						type = "toggle",
						get = function() return bb.groups[7] end,
						set = function(info, value) bb.groups[7] = value end,
						order = 17,
					},
					group8 = {
						name = L["Group 8"],
						type = "toggle",
						get = function() return bb.groups[8] end,
						set = function(info, value) bb.groups[8] = value end,
						order = 18,
					},
				},
			}, --groups

			output = {
				name = L["Default Output"],
				desc = L["Set the default check output."],
				type = "group",
				inline = true,
				get = function(info)
					local key = info[#info]
					return string.gsub(db.outputDefault, "(.-):(.*)", "%1") == key
				end,
				set = function(info, value)
					if value then
						local key = info[#info]
						db.outputDefault = key
					end
				end,
				order = 20,
				args = {
					self = {
						name = L["Self"],
						desc = L["Print to your chat window."],
						type = "toggle",
						order = 5,
					},
					say = {
						name = L["Say"],
						desc = L["Report to say."],
						type = "toggle",
						order = 8,
					},
					group = {
						name = L["Group"],
						desc = L["Report to party or raid chat."],
						type = "toggle",
						order = 10,
					},
					party = {
						name = L["Party"],
						desc = L["Report to party chat."],
						type = "toggle",
						order = 11,
					},
					raid = {
						name = L["Raid"],
						desc = L["Report to raid chat."],
						type = "toggle",
						order = 12,
					},
					raid_warning = {
						name = L["Raid Warning"],
						desc = L["Report as a raid warning."],
						type = "toggle",
						order = 13,
					},
					guild = {
						name = L["Guild"],
						desc = L["Report to guild chat."],
						type = "toggle",
						order = 15,
					},
					officer = {
						name = L["Officer"],
						desc = L["Report to officer chat."],
						type = "toggle",
						order = 20,
					},
					channel = {
						name = L["Channel"],
						desc = L["Report to a custom channel."],
						type = "toggle",
						order = 25,
					},
					extra = { --pretty silly implementation, but oh well
						name = L["Channel"],
						type = "select",
						values = function() BigBrother:GetChatChannels() end,
						get = function()
							local channel = string.gsub(db.outputDefault, "(.-):(.*)", "%2")
							return GetChannelName(channel) > 0 and channel or ""
						end,
						set = function(info, value)
							db.outputDefault = "channel:"..value
						end,
						hidden = function() return string.gsub(db.outputDefault, "(.-):(.*)", "%1") ~= "channel" end,
						order = 101,
					},
				},
			},

			blame = {
				name = L["Whisper Missing"],
				desc = L["Whisper players that are missing consumables on checks."],
				type = "toggle",
				order = 25,
			},
			checkReadyCheck = {
				name = L["Check on Ready Check"],
				desc = L["Buff check on a ready check when in a non-LFG instance group.\n\n|cffffff33Checked|r: Report if started by you.\n|cffccccccGray|r: Report always."],
				type = "toggle",
				tristate = true,
				-- unchecked (false/0), checked (true/1), greyed (nil/2)
				get = function()
					local value = db.checkReadyCheck
					if value == 0 then
						return false
					elseif value == 1 then
						return true
					elseif value == 2 then
						return nil
					end
				end,
				set = function(info, value)
					if value == false then
						value = 0
					elseif value == true then
						value = 1
					elseif value == nil then
						value = 2
					end
					db.checkReadyCheck = value
				end,
				order = 30,
			},
			maxOnly = {
				name = L["Max Quality Only"],
				desc = L["Only count consumables of the highest available stat value in checks."],
				type = "toggle",
				order = 35,
			},
		},
	}
end

